#ifndef NRF52_H
#define NRF52_H

#include <stdint.h>

/*******************************
 * CMSIS Definitions for NRF52 *
 *******************************/

// Interrupt Number Definition
typedef enum IRQn {
	/* Cortex-M4 Processor Exception Numbers */
	Reset_IRQn = -15, // 1 - Reset Vector - Invoked on power up and warm reset.
	NonMaskableInt_IRQn = -14, // 2 - Non-Maskable Interrupt - Cannot be stopped or preempted.
	HardFault_IRQn = -13, // 3 - Hard Fault - All classes of fault.
	MemoryManagement_IRQn = -12, // 4 - Memory Management - MPU mismatch including Access Violation and No Match.
	BusFault_IRQn = -11, // 5 - Bus Fault - Pre-Fetch, Memory Access Fault, and other address/memory related faults.
	UsageFault_IRQn = -10, // 6 - Usage Fault - Undefined Instruction, Illegal State Transition.
	SVCall_IRQn = -5, // 11 - System Service Call via SVC Instruction
	DebugMonitor_IRQn = -4, // 12 - Debug Monitor
	PendSV_IRQn = -2, // 14 - Pendable Request for System Service
	SysTick_IRQn = -1, // 15 - System Tick Timer
	/* nRF52 Specific Exception Numbers */
	PowerClk_IRQn = 0, // Power/Clock Controller
	Radio_IRQn = 1, // Radio Controller
	UART0_IRQn = 2, // UART0
	SPI0_TWI0_IRQn = 3, // SPI0/TWI0 Controller
	SPI1_TWI1_IRQn = 4, // SPI1/TWI1 Controller
	NFC_IRQn = 5, // Near-Field Communication Controller
	GPIOTE_IRQn = 6, // General Purpose Input Output Task and Events
	SAADC_IRQn = 7, // Successive Approximation Analog-to-Digital Converter
	Timer0_IRQn = 8, // Timer 0
	Timer1_IRQn = 9, // Timer 1
	Timer2_IRQn = 10, // Timer 2
	RTC0_IRQn = 11, // Real Time Clock 0
	Temp_IRQn = 12, // Temperature Sensor
	RNG_IRQn = 13, // Random Number Generator
	ECB_IRQn = 14, // AES Electronic Code Book
	CCM_AAR_IRQn = 15, // AES CCM
	WDT_IRQn = 16, // Watchdog Timer
	RTC1_IRQn = 17, // Real Time Clock 1
	QDEC_IRQn = 18, // Quadrature Decoder
	COMP_LPCOMP_IRQn = 19, // Comparator/Low Power Comparator
	SWI0_EGU0_IRQn = 20, // SWI0/EGU0
	SWI1_EGU1_IRQn = 21, // SWI1/EGU1
	SWI2_EGU2_IRQn = 22, // SWI2/EGU2
	SWI3_EGU3_IRQn = 23, // SWI3/EGU3
	SWI4_EGU4_IRQn = 24, // SWI4/EGU4
	SWI5_EGU5_IRQn = 25, // SWI5/EGU5
	Timer3_IRQn = 26, // Timer 3
	Timer4_IRQn = 27, // Timer 4
	PWM0_IRQn = 28, // Pulse-Width Modulation Controller 0
	PDM_IRQn = 29, // Pulse-Density Modulation Controller
	MWU_IRQn = 32, // Memory Watch Unit
	PWM1_IRQn = 33, // Pulse-Width Modulation Controller 1
	PWM2_IRQn = 34, // Pulse-Width Modulation Controller 2
	SPI2_IRQn = 35, // Serial Peripheral Interface 2
	RTC2_IRQn = 36, // Real Time Clock 2
	I2S_IRQn = 37, // Inter-IC Sound Controller
	FPU_IRQn = 38 // Floating-Point Unit
} IRQn_t;

typedef struct DeviceVectors {
	/* Stack Pointer */
	void* pvStack;

	/* Cortex-M4 Handlers */
	void* pfnReset_Handler;
	void* pfnNMI_Handler;
	void* pfnHardFault_Handler;
	void* pfnMemManage_Handler;
	void* pfnBusFault_Handler;
	void* pfnUsageFault_Handler;
	void* pfnReserved1_Handler;
	void* pfnReserved2_Handler;
	void* pfnReserved3_Handler;
	void* pfnReserved4_Handler;
	void* pfnSVC_Handler;
	void* pfnDebugMon_Handler;
	void* pfnReserved5_Handler;
	void* pfnPendSV_Handler;
	void* pfnSysTick_Handler;

	/* Peripheral Handlers */
	void* pfnPowerClk_Handler; // 0 - Power/Clock Controller
	void* pfnRadio_Handler; // 1 - Radio Controller
	void* pfnUART0_Handler; // 2 - UART0
	void* pfnSPI0_TWI0_Handler; // 3 - SPI0/TWI0 Controller
	void* pfnSPI1_TWI1_Handler; // 4 - SPI1/TWI1 Controller
	void* pfnNFC_Handler; // 5 - Near-Field Communication Controller
	void* pfnGPIOTE_Handler; // 6 - General Purpose Input Output Task and Events
	void* pfnSAADC_Handler; // 7 - Successive Approximation Analog-to-Digital Converter
	void* pfnTimer0_Handler; // 8 - Timer 0
	void* pfnTimer1_Handler; // 9 - Timer 1
	void* pfnTimer2_Handler; // 10 - Timer 2
	void* pfnRTC0_Handler; // 11 - Real Time Clock 0
	void* pfnTemp_Handler; // 12 - Temperature Sensor
	void* pfnRNG_Handler; // 13 - Random Number Generator
	void* pfnECB_Handler; // 14 - AES Electronic Code Book
	void* pfnCCM_AAR_Handler; // 15 - AES CCM
	void* pfnWDT_Handler; // 16 - Watchdog Timer
	void* pfnRTC1_Handler; // 17 - Real Time Clock 1
	void* pfnQDEC_Handler; // 18 - Quadrature Decoder
	void* pfnCOMP_LPCOMP_Handler; // 19 - Comparator/Low Power Comparator
	void* pfnSWI0_EGU0_Handler; // 20 - SWI0/EGU0
	void* pfnSWI1_EGU1_Handler; // 21 - SWI1/EGU1
	void* pfnSWI2_EGU2_Handler; // 22 - SWI2/EGU2
	void* pfnSWI3_EGU3_Handler; // 23 - SWI3/EGU3
	void* pfnSWI4_EGU4_Handler; // 24 - SWI4/EGU4
	void* pfnSWI5_EGU5_Handler; // 25 - SWI5/EGU5
	void* pfnTimer3_Handler; // 26 - Timer 3
	void* pfnTimer4_Handler; // 27 - Timer 4
	void* pfnPWM0_Handler; // 28 - Pulse-Width Modulation Controller 0
	void* pfnPDM_Handler; // 29 - Pulse-Density Modulation Controller
	void* pfnReserved6_Handler;
	void* pfnReserved7_Handler;
	void* pfnMWU_Handler; // 32 - Memory Watch Unit
	void* pfnPWM1_Handler; // 33 - Pulse-Width Modulation Controller 1
	void* pfnPWM2_Handler; // 34 - Pulse-Width Modulation Controller 2
	void* pfnSPI2_Handler; // 35 - Serial Peripheral Interface 2
	void* pfnRTC2_Handler; // 36 - Real Time Clock 2
	void* pfnI2S_Handler; // 37 - Inter-IC Sound Controller
	void* pfnFPU_Handler; // 38 - Floating-Point Unit
} DeviceVectors_t;

/* Cortex-M4 Core Handlers */
void Reset_Handler(void);
void NMI_Handler(void);
void HardFault_Handler(void);
void MemManage_Handler(void);
void BusFault_Handler(void);
void UsageFault_Handler(void);
void SVC_Handler(void);
void DebugMon_Handler(void);
void PendSV_Handler(void);
void SysTick_Handler(void);

/* Peripheral Handlers */
void PowerClk_Handler(void);
void Radio_Handler(void);
void UART0_Handler(void);
void SPI0_TWI0_Handler(void);
void SPI1_TWI1_Handler(void);
void NFC_Handler(void);
void GPIOTE_Handler(void);
void SAADC_Handler(void);
void Timer0_Handler(void);
void Timer1_Handler(void);
void Timer2_Handler(void);
void RTC0_Handler(void);
void Temp_Handler(void);
void RNG_Handler(void);
void ECB_Handler(void);
void CCM_AAR_Handler(void);
void WDT_Handler(void);
void RTC1_Handler(void);
void QDEC_Handler(void);
void COMP_LPCOMP_Handler(void);
void SWI0_EGU0_Handler(void);
void SWI1_EGU1_Handler(void);
void SWI2_EGU2_Handler(void);
void SWI3_EGU3_Handler(void);
void SWI4_EGU4_Handler(void);
void SWI5_EGU5_Handler(void);
void Timer3_Handler(void);
void Timer4_Handler(void);
void PWM0_Handler(void);
void PDM_Handler(void);
void MWU_Handler(void);
void PWM1_Handler(void);
void PWM2_Handler(void);
void SPI2_Handler(void);
void RTC2_Handler(void);
void I2S_Handler(void);
void FPU_Handler(void);

/** CMSIS Includes **/
#include <core_cm4.h>
#include "system.h"

/** SOFTWARE PERIPHERAL API DEFINITION FOR NRF52 **/
#include "component/component_clock.h"
#include "component/component_ficr.h"
#include "component/component_gpio.h"
#include "component/component_power.h"
#include "component/component_rtc.h"
#include "component/component_temp.h"
#include "component/component_timer.h"

/** BASE ADDRESS DEFINITIONS FOR NRF52 **/
#define FICR ((FICR_t*) 0x10000000UL)
#define POWER ((Power_t*) 0x40000000UL)
#define CLOCK ((Clock_t*) 0x40000000UL)
#define TEMP ((Temp_t*) 0x4000C000UL)
#define TIMER2 ((Timer_t*) 0x4000A000UL)
#define RTC1 ((RTC_t*) 0x40011000UL)
#define GPIO ((GPIO_t*) 0x50000000UL)

#endif
